<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use FM\Traits\TQuery;

class Fm00010000ParametrizacaoGeralFactory
{

    use TQuery;

    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function postMascara($params)
    {
        $sql = "INSERT INTO FCM_PARAMGERAL (NRPARAMGERAL, IDOBRIGACDLOCAT, IDUTILIZAMASCCDLOCAT, DSMASCCDLOCAT, IDALTERAMASCCDLOCAT, IDATIVO,
                                            NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO, DTULTATU, CDOPERULTATU, NRORGULTATU)
                VALUES (:NRPARAMGERAL, :IDOBRIGACDLOCAT, :IDUTILIZAMASCCDLOCAT, :DSMASCCDLOCAT, :IDALTERAMASCCDLOCAT, 'S',
                         :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO, SYSDATE, :CDOPERULTATU, :NRORGULTATU)";
        $this->connection->executeStatement($sql, $params);
    }

    public function updateConflitoParametrizacao($params)
    {
        $sql = "UPDATE FCM_PARAMGERAL 
                   SET CDOPERULTATU = :CDOPERULTATU,
                       IDVALIDACRIATAREFA = :IDVALIDACRIATAREFA,
                       DTULTATU = SYSDATE
                 WHERE NRPARAMGERAL = :NRPARAMGERAL";

        $this->connection->executeStatement($sql, $params);
    }

    public function getParametroConflito()
    {
        $sql = "SELECT IDVALIDACRIATAREFA 
                  FROM FCM_PARAMGERAL 
                 WHERE NRPARAMGERAL = 59";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function getMascara()
    {

        $sql = "SELECT IDOBRIGACDLOCAT, IDUTILIZAMASCCDLOCAT, DSMASCCDLOCAT, IDALTERAMASCCDLOCAT 
                FROM FCM_PARAMGERAL";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function updateMascara($params)
    {
        $sql = "UPDATE FCM_PARAMGERAL 
                SET IDOBRIGACDLOCAT = :IDOBRIGACDLOCAT,
                IDUTILIZAMASCCDLOCAT = :IDUTILIZAMASCCDLOCAT,
                DSMASCCDLOCAT = : DSMASCCDLOCAT,
                IDALTERAMASCCDLOCAT = : IDALTERAMASCCDLOCAT,
                NRORGINCLUSAO = :NRORGINCLUSAO,
                DTULTATU = SYSDATE,
                CDOPERULTATU = :CDOPERULTATU,
                NRORGULTATU = :NRORGULTATU";

        $this->connection->executeStatement($sql, $params);
    }

    public function getCountMascara()
    {

        $sql = "SELECT COUNT(NRPARAMGERAL) AS QTA 
                FROM FCM_PARAMGERAL";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    // INTEGRAÇÃO COM SM
    public function validarItem($parametros)
    {

        $sql = "SELECT COUNT(*) CONT
                    FROM FCM_RELSERVITEMTAR
                    WHERE CDSERVORCA = :CDSERVORCA
                    AND CDTPMEDIDASERV = :CDTPMEDIDASERV
                    AND NRATIVIDADE = :NRATIVIDADE
                    AND IDATIVO = 'S'";

        $retorno = $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
        return $retorno[0]['CONT'];
    }

    public function inserirRelacionamento($parametros)
    {

        $sql = "INSERT INTO FCM_RELSERVITEMTAR(NRRELSERVITEMTAR, CDSERVORCA, CDTPMEDIDASERV, NRATIVIDADE, IDATIVO, 
                                               NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
                VALUES (:NRRELSERVITEMTAR, :CDSERVORCA, :CDTPMEDIDASERV, :NRATIVIDADE, 'S' , :NRORG, SYSDATE, 
                        :CDOPERINCLUSAO, :NRORGINCLUSAO)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getRelacionamentoServicoTipoMedida()
    {

        $sql = "SELECT  ROWNUM AS ID, RELACIONAMENTO.* 
                    FROM (SELECT DISTINCT FCM_RELSERVITEMTAR.CDSERVORCA, FCT_SERVICOORCA.NMSERVORCA, FCM_RELSERVITEMTAR.CDTPMEDIDASERV, FCT_TIPOMEDIDASERV.DSTPMEDIDASERV
                    FROM FCM_RELSERVITEMTAR
                    JOIN FCT_TIPOMEDIDASERV
                        ON FCT_TIPOMEDIDASERV.CDTPMEDIDASERV = FCM_RELSERVITEMTAR.CDTPMEDIDASERV
                    JOIN FCT_SERVICOORCA
                        ON FCT_SERVICOORCA.CDSERVORCA = FCM_RELSERVITEMTAR.CDSERVORCA
                        WHERE FCM_RELSERVITEMTAR.IDATIVO = 'S') RELACIONAMENTO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getItensRelacionamentoServicoTipoMedida($parametros)
    {

        $sql = "SELECT FCM_RELSERVITEMTAR.NRRELSERVITEMTAR, FCM_ATIVIDADE.NRATIVIDADE, FCM_ATIVIDADE.NMATIVIDADE 
                        FROM FCM_RELSERVITEMTAR
                        JOIN FCM_ATIVIDADE 
                          ON FCM_ATIVIDADE.NRATIVIDADE = FCM_RELSERVITEMTAR.NRATIVIDADE
                       WHERE FCM_RELSERVITEMTAR.CDTPMEDIDASERV = :CDTPMEDIDASERV
                         AND FCM_RELSERVITEMTAR.CDSERVORCA = :CDSERVORCA
                         AND FCM_RELSERVITEMTAR.IDATIVO = 'S'";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function inativarServicos($parametros)
    {

        $sql = "UPDATE FCM_RELSERVITEMTAR
                    SET IDATIVO = 'N',
                        DTULTATU = SYSDATE,
                        CDOPERULTATU =:CDOPERULTATU,
                        NRORGULTATU = :NRORGULTATU
                    WHERE CDSERVORCA = :CDSERVORCA
                    AND CDTPMEDIDASERV = :CDTPMEDIDASERV";

        $this->connection->executeStatement($sql, $parametros);
    }
}
