<?php

namespace FM\Service;

use FM\Factory\FuncionariosHierarqFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class FuncionariosHierarqService
{

    protected $connection;
    protected $FuncionariosHierarqFactory;
    protected $Environment;
    protected $newCode;

    public function __construct(EntityManager $entityManager, FuncionariosHierarqFactory $FuncionariosHierarqFactory, Environment $Environment, NewCode $newCode)
    {
        $this->connection                   = $entityManager->getConnection();
        $this->FuncionariosHierarqFactory   = $FuncionariosHierarqFactory;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
    }

    // Busca toda a ramificação de usuários da hierarquia
    public function getHierarchyDetails($nrhierarquiafunc)
    {
        return $this->FuncionariosHierarqFactory->getHierarchyDetails($nrhierarquiafunc);
    }

    public function montarHierarquiaOperadores($cdOperador)
    {
        $listaOperadores = null;

        $params = array(
            "CDOPERADOR" => $cdOperador
        );

        $operadores = $this->FuncionariosHierarqFactory->getHierarquiaFacilities($params);
        
        foreach ($operadores as $operador) {
            if ($listaOperadores != null) {
                $listaOperadores = $listaOperadores . ",";
            }
            $listaOperadores = $listaOperadores . "'" . $operador['CDOPERADOR'] . "'";
        }

        return $listaOperadores;
    }

    public function getHierarquiaFacilities($parametros)
    {
        $param = array(
            "CDOPERADOR" => $parametros['CDOPERADOR']
        );

        return $this->FuncionariosHierarqFactory->getHierarquiaFacilities($param);
    }

    // Busca uma lista personalizada de usuários que podem ser adicionados na hierarquia
    public function getHierarchyUsers($nrhierarquiafunc)
    {
        return $this->FuncionariosHierarqFactory->getHierarchyUsers($nrhierarquiafunc);
    }

    // Busca uma lista personalizada de usuários que podem ser Pais na hierarquia
    public function getHierarchyUsersParents($nrhierarquiafunc)
    {

        return $this->FuncionariosHierarqFactory->getHierarchyUsersParents($nrhierarquiafunc);
    }

    // Insere um novo usuário à hierarquia
    public function insertHierarchyUser($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrfuncionarioshierarq = $this->newCode->getCodeByTableName('FCM_FUNCIONARIOSHIERARQ', $this->Environment->getNrOrg(), 1, 10, true);

            $paramsUsers = array(
                'NRFUNCIONARIOSHIERARQ' => $nrfuncionarioshierarq,
                'NRHIERARQUIAFUNC'      => $params['params']['NRHIERARQUIAFUNC'],
                'CDOPERADOR'            => $params['params']['CDOPERADOR'],
                'CDOPERADORPAI'         => $params['params']['CDOPERADORPAI']
            );

            $this->FuncionariosHierarqFactory->insertHierarchyUser($paramsUsers);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    // Deleta um usuário e sua ramificação abaixo
    public function deleteHierarchyUser($params)
    {
        $this->connection->beginTransaction();

        try {
            foreach ($params['params'] as $cdoperador) {
                $this->FuncionariosHierarqFactory->deleteHierarchyUser($cdoperador);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}
