<?php

namespace FM\Service;

use FM\Factory\HierarquiaUsuariosFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class HierarquiaUsuariosService
{

    protected $HierarquiaUsuariosFactory;
    protected $Environment;
    protected $newCode;

    public function __construct(EntityManager $entityManager, HierarquiaUsuariosFactory $HierarquiaUsuariosFactory, Environment $Environment, NewCode $newCode)
    {
        $this->connection                                       = $entityManager->getConnection();
        $this->HierarquiaUsuariosFactory                        = $HierarquiaUsuariosFactory;
        $this->Environment                                      = $Environment;
        $this->newCode                                          = $newCode;
    }

    // Busca as hierarquias cadastradas
    public function getHierarquia()
    {
        return $this->HierarquiaUsuariosFactory->getHierarquia();
    }

    //Busca uma lista com os usuários cadastrados na hierarquia escolhida
    public function getHierarquiaDetalhes($parametros)
    {
        $param = array(
            "NRHIERARQUIAFUNC" => $parametros['NRHIERARQUIAFUNC']
        );
        return $this->HierarquiaUsuariosFactory->getHierarquiaDetalhes($param);
    }

    public function getHierarquiaFacilities($parametros)
    {
        $param = array(
            "NRFUNCIONARIO" => $parametros['NRFUNCIONARIO']
        );

        return $this->HierarquiaUsuariosFactory->getHierarquiaFacilities($param);
    }

    // Busca uma lista personalizada de usuários que podem ser adicionados na hierarquia
    public function getHierarquiaUsuarios($params)
    {
        $param = array(
            "NRHIERARQUIAFUNC" => $params["find"]['NRHIERARQUIAFUNC']
        );
        return $this->HierarquiaUsuariosFactory->getHierarquiaUsuarios($param);
    }

    // Busca uma lista personalizada de usuários que podem ser adicionados na hierarquia
    public function getHierarquiaPais($params)
    {
        $param = array(
            "NRHIERARQUIAFUNC" => $params["find"]['NRHIERARQUIAFUNC']
        );
        return $this->HierarquiaUsuariosFactory->getHierarquiaPais($param);
    }
    // Função para editar ou inserir nora hierarquia
    public function salvarHierarquia($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();

            foreach ($params["DRG"] as $row) {
                if ($row["NRHIERARQUIAFUNC"] != 0) {
                    $parametros = array(
                        "NRHIERARQUIAFUNC"      => $row["NRHIERARQUIAFUNC"],
                        "IDATIVO"               => $row["IDATIVO"],
                        "NMHIERARQUIAFUNC"      => (isset($row['NMHIERARQUIAFUNC']) ? $row['NMHIERARQUIAFUNC'] : ""),
                        "NRORGULTATU"           => $nrorg,
                        "CDOPERULTATU"          => $operador
                    );

                    $this->HierarquiaUsuariosFactory->editarHierarquia($parametros);
                } else {
                    $NRHIERARQUIAFUNC = $this->newCode->getCodeByTableName("FCM_HIERARQUIAFUNC", $nrorg, 1, 10, true);
                    $parametros = array(
                        "NRHIERARQUIAFUNC"      => $NRHIERARQUIAFUNC,
                        "IDATIVO"               => (isset($row['IDATIVO']) ? $row['IDATIVO'] : ""),
                        "NMHIERARQUIAFUNC"      => (isset($row['NMHIERARQUIAFUNC']) ? $row['NMHIERARQUIAFUNC'] : ""),
                        "NRORG"                 => $nrorg,
                        "NRORGINCLUSAO"         => $nrorg,
                        "CDOPERINCLUSAO"        => $operador,
                        "NRORGULTATU"           => $nrorg,
                        "CDOPERULTATU"          => $operador
                    );
                    $this->HierarquiaUsuariosFactory->salvarHierarquia($parametros);
                }
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    // Insere um novo usuário à hierarquia

    public function inserirUsuarios($params)
    {

        $this->connection->beginTransaction();
        $nrorg = $this->Environment->getNrorg();
        $operador = $this->Environment->getCdOperador();

        // debugApi([$parametros]);

        try {
            foreach ($params['CDOPERADOR'] as $operador) {
                $nrfuncionarioshierarq =  $this->newCode->getCodeByTableName("FCM_FUNCIONARIOSHIERARQ", $nrorg, 1, 10, true);
                $paramsUsers = array(
                    'NRFUNCIONARIOSHIERARQ' => $nrfuncionarioshierarq,
                    "NRHIERARQUIAFUNC"      => $params['NRHIERARQUIAFUNC'],
                    'CDOPERADOR'            => $operador,
                    'CDOPERADORPAI'         => $params['CDOPERADORPAI'],
                    "NRORG"                 => $nrorg,
                    "NRORGINCLUSAO"         => $nrorg,
                    "CDOPERINCLUSAO"        => $operador
                );
                $this->HierarquiaUsuariosFactory->inserirUsuarios($paramsUsers);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    // Deleta a hierarquia escolhida
    public function excluirHierarquia($parametro)
    {
        $this->connection->beginTransaction();
        try {
            $param = implode(',', $parametro["NRS"]);
            $this->HierarquiaUsuariosFactory->excluirHierarquia($param);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    // Função responsável por retornar o array do PAI e de todos os filhos.
    public function selectFilhos($parametro)
    {
        $string = (implode(',', $parametro)); // transforma em string o array para concatenar com a query
        $retorno = $this->HierarquiaUsuariosFactory->buscarCdoperador($string); // Armazena na variavel retorno os valores dos filhos retornados

        if (empty($retorno)) { // verifica se o retorno não é vazio caso seja encerra o chamado da função devolvendo o parametro recebido
            return $parametro;
        }
        return $this->selectFilhos(array_merge($parametro, $this->getCdOeprador($retorno))); // faz uma chama recursiva da propria função passando como parâmetro
    }                                                                                        // os parametros recebidos junto com os filhos encontrados

    // Recebe o retorno do banco de dados e transforma em um array
    public function getCdOeprador($v)
    {
        $aux = []; // recebe os valores retornados
        foreach ($v as $cd) { // itera sobre os valores do BD
            $aux[] = $cd["CDOPERADOR"]; // aux recebe de cada elemento o valor indexo pela chave CDOPERADOR
        }
        return $aux; // retorna um novo array
    }

    // Deleta um ou mais usuários da hierarquia
    public function excluirUsuarios($parametro)
    {
        $this->connection->beginTransaction();

        $NRGERAL =  $this->selectFilhos($parametro["NRS"]); // armazena o NRGERAL o NR recebido e os filhos encontrados
        try {
            $param = implode(',', $NRGERAL);
            $this->HierarquiaUsuariosFactory->excluirUsuarios($param);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    // public function getHierarquiaFunc()
    // {
    //     return $this->HierarquiaUsuariosFactory->getHierarquiaFunc();
    // }
}
